-- BuffCheck_Settings.lua
-- by Mithrandir1210

--[[
	TO DO:
		- Clean up else block of SetAllBuffCheckbuttons()
		- Make addon-local variables local where possible

]]

-- GENERAL INFORMATION --======================================================

-- Load addon-local variables table
local BuffCheck_Name, bc = ...
	-- BuffCheck_Name = "BuffCheck"
	-- BuffCheck = {}

bc.Description = {
	Title = GetAddOnMetadata(BuffCheck_Name, "Title"),
	Version = GetAddOnMetadata(BuffCheck_Name, "Version"),
	Author = GetAddOnMetadata(BuffCheck_Name, "Author")
}

-- DEFAULT SETTINGS --=========================================================

-- COLOR PREFIXES
bc.Colors = {
	Red = "|cFFFF0000",
	Blue = "|cFF0000FF",
	Green = "|cFF008000",
	Yellow = "|cFFFFFF00",
	Brown = "|cFF663300",
	Gray = "|cFF808080",
	White = "|cFFFFFFFF",
	Black = "|cFF000000",
	Aquablue = "|cFF00B8E6"
}

bc.Defaults = {
	FontColor = bc.Colors.Aquablue,
	DisableBuffOutput = false,
	AnnounceMissing = false,
	CheckAfterReadycheck = false,
	DisplaySettingsOnLoad = true,

	BuffStatus = {
		Food = true,
		GreaterFlasks = true,
		Stats = true,
		Stam = true,
		Ap = true,
		Sp = true,
		Mastery = true,
		Haste = true,
		Crit = true,
		Multi = true,
		Vers = true,
		Rune = true
	}
}

bc.NumPlayers = nil

-- INITIALIZATION --===========================================================

-- CREATE FRAMES --
bc.InitEventFrame = CreateFrame("Frame")
bc.MainEventFrame = CreateFrame("Frame")

-- REGISTER EVENTS --
bc.InitEventFrame:RegisterEvent("ADDON_LOADED")
bc.MainEventFrame:RegisterEvent("READY_CHECK_FINISHED")
bc.MainEventFrame:RegisterEvent("CHAT_MSG_ADDON")

-- POPULATE INTERFACE OPTIONS --===============================================

-- Add entry to interface options
bc.panel = CreateFrame ("Frame", "bc_Panel", UIParent);
bc.panel.name = "Buff Check";
InterfaceOptions_AddCategory(bc.panel);

-- Create main title (header)
bc.SettingsTitle = bc.panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
bc.SettingsTitle:SetPoint("TOPLEFT", 10, -15)
bc.SettingsTitle:SetText("Buff Check")

-- Create subtitle
bc.SettingsTitle = bc.panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
bc.SettingsTitle:SetPoint("TOPLEFT", 10, -50)
bc.SettingsTitle:SetText("General Options")

-- Button Tables
bc.Btn = {

	Settings = {
		DisableBuffOutput,
		Announce,
		Readycheck,
		DisplaySettingsOnLoad,
		ConsolidateOutput,
	
		Buffs = {
			Food,
			GreaterFlasks,
			Stats,
			Stam,
			Ap,
			Sp,
			Mastery,
			Haste,
			Crit,
			Multi,
			Vers,
			Rune
		},
		
		EnableAllBuffs
	}
	
}

-- UI METHODS --===============================================================

local CreateSettingsCheckbtn = function(label, btnFrameAnchor, x, y, txtFrameAnchor)
	
	local Btn = CreateFrame("CheckButton", nil, bc.panel, "InterfaceOptionsCheckButtonTemplate")
	Btn:SetPoint(btnFrameAnchor, x, y)
	local Btn_Text = Btn:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	
	if (txtFrameAnchor == "LEFT" or txtFrameAnchor == "TOPLEFT") then
		Btn_Text:SetPoint(txtFrameAnchor, Btn, "RIGHT", 0, -1)	
	elseif (txtFrameAnchor == "TOPRIGHT" or txtFrameAnchor == "RIGHT") then
		Btn_Text:SetPoint(txtFrameAnchor, Btn, "LEFT", 0, 1)
	else
		Btn_Text:SetPoint(txtFrameAnchor, Btn, "RIGHT", 0, -1)
	end
	
	Btn_Text:SetText(label)
	
	return Btn, Btn_Text
end

-- Create checkbuttons and their labels

-- 1. Disable output?
bc.Btn.Settings.DisableBuffOutput = CreateSettingsCheckbtn("Disable buff output", "TOPLEFT", 20, -70, "LEFT")

-- 2. Announce to raid?
bc.Btn.Settings.Announce = CreateSettingsCheckbtn("Announce missing buffs to chat", "TOPLEFT", 20, -100, "LEFT")

-- 3. Check after readycheck?
bc.Btn.Settings.Readycheck = CreateSettingsCheckbtn("Check for buffs after readycheck", "TOPLEFT", 20, -130, "LEFT")

-- 4. Display settings on load?
bc.Btn.Settings.DisplaySettingsOnLoad = CreateSettingsCheckbtn("Display settings on load", "TOPLEFT", 20, -160, "LEFT")

-- 5. Display number of players missing buffs instead of names
bc.Btn.Settings.ConsolidateOutput = CreateSettingsCheckbtn("Consolidate output", "TOPLEFT", 20, -190, "LEFT")

-- 6. ENABLE/DISABLE BUFFS? --

-- Create subtitle
bc.SettingsTitle = bc.panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
bc.SettingsTitle:SetPoint("TOPRIGHT", -10, -50)
bc.SettingsTitle:SetText("Enabled Buffs")

-- Food
bc.Btn.Settings.Buffs.Food = CreateSettingsCheckbtn("Food", "TOPRIGHT", -20, -70, "RIGHT")

-- GreaterFlasks
bc.Btn.Settings.Buffs.GreaterFlasks = CreateSettingsCheckbtn("Greater Flasks", "TOPRIGHT", -20, -100, "RIGHT")

-- Stats
bc.Btn.Settings.Buffs.Stats = CreateSettingsCheckbtn("Stats", "TOPRIGHT", -20, -130, "RIGHT")

-- Stam
bc.Btn.Settings.Buffs.Stam = CreateSettingsCheckbtn("Stamina", "TOPRIGHT", -20, -160, "RIGHT")

-- Ap
bc.Btn.Settings.Buffs.Ap = CreateSettingsCheckbtn("Attack Power", "TOPRIGHT", -20, -190, "RIGHT")

-- Sp
bc.Btn.Settings.Buffs.Sp = CreateSettingsCheckbtn("Spell Power", "TOPRIGHT", -20, -220, "RIGHT")

-- Mastery
bc.Btn.Settings.Buffs.Mastery = CreateSettingsCheckbtn("Mastery", "TOPRIGHT", -20, -250, "RIGHT")

-- Haste
bc.Btn.Settings.Buffs.Haste = CreateSettingsCheckbtn("Haste", "TOPRIGHT", -20, -280, "RIGHT")

-- Crit
bc.Btn.Settings.Buffs.Crit = CreateSettingsCheckbtn("Critical Strike", "TOPRIGHT", -20, -310, "RIGHT")

-- Multi
bc.Btn.Settings.Buffs.Multi = CreateSettingsCheckbtn("Multistrike", "TOPRIGHT", -20, -340, "RIGHT")

-- Vers
bc.Btn.Settings.Buffs.Vers = CreateSettingsCheckbtn("Versatility", "TOPRIGHT", -20, -370, "RIGHT")

-- AugmentRune
bc.Btn.Settings.Buffs.Rune = CreateSettingsCheckbtn("Augment Rune", "TOPRIGHT", -20, -400, "RIGHT")

-- Toggle All Buffs
bc.Btn.Settings.EnableAllBuffs = CreateFrame("Button", nil, bc.panel, "UIPanelButtonTemplate")
bc.Btn.Settings.EnableAllBuffs:SetSize(100,22)
bc.Btn.Settings.EnableAllBuffs:SetText("Toggle All")
bc.Btn.Settings.EnableAllBuffs:SetPoint("TOPRIGHT", -20, -440)

-------------------------------------------------------------------------------
-- Set state of ALL Enabled Buffs Checkbuttons to specified argument.
-- If 'status' is a boolean, set all buttons to the same status
-- Otherwise, set to values of BuffCheck_ActiveSettings.BuffStatus
-------------------------------------------------------------------------------
bc.SetAllBuffCheckbuttons = function (status)

	if type(status) == "boolean" then
		for i in pairs(bc.Btn.Settings.Buffs) do
			bc.Btn.Settings.Buffs[i]:SetChecked(status)	
			 -- test this***
		end
		for j in pairs(bc.Btn.Settings.Buffs) do
			BuffCheck_ActiveSettings.BuffStatus[j] = status	
			 -- test this***
		end
	else
		-- Find a way to clean this up a bit
		for i in pairs(BuffCheck_ActiveSettings.BuffStatus) do
			bc.Btn.Settings.Buffs.Food:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Food)
			bc.Btn.Settings.Buffs.GreaterFlasks:SetChecked(BuffCheck_ActiveSettings.BuffStatus.GreaterFlasks)
			bc.Btn.Settings.Buffs.Stats:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Stats)
			bc.Btn.Settings.Buffs.Stam:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Stam)
			bc.Btn.Settings.Buffs.Ap:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Ap)
			bc.Btn.Settings.Buffs.Sp:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Sp)
			bc.Btn.Settings.Buffs.Mastery:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Mastery)
			bc.Btn.Settings.Buffs.Haste:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Haste)
			bc.Btn.Settings.Buffs.Crit:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Crit)
			bc.Btn.Settings.Buffs.Multi:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Multi)
			bc.Btn.Settings.Buffs.Vers:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Vers)
			bc.Btn.Settings.Buffs.Rune:SetChecked(BuffCheck_ActiveSettings.BuffStatus.Rune)	

		end
	end
	
end

-- OTHER METHODS --============================================================

-------------------------------------------------------------------------------
-- Print the current general settings to the console (void).
-------------------------------------------------------------------------------
bc.ViewSettings = function()

	-- 1. Disable output
	-- if BuffCheck_ActiveSettings.DisableBuffOutput then
		-- bc.Display("Disable buff output?: ".."Yes")
	-- else
		-- bc.Display("Disable buff output?: ".."No")
	-- end
	
	-- 2. Announce missing 
	if BuffCheck_ActiveSettings.AnnounceMissing then
		bc.Display("Announce to chat?: ".."Yes")
	else
		bc.Display("Announce to chat?: ".."No")
	end
	
	--3. Check after readycheck
	-- if BuffCheck_ActiveSettings.CheckAfterReadycheck then
		-- bc.Display("Check after readycheck?: ".."Yes")
	-- else
		-- bc.Display("Check after readycheck?: ".."No")
	-- end
	
end

-------------------------------------------------------------------------------
-- Reset settings to their defaults (void).
-------------------------------------------------------------------------------
bc.SetDefaults = function()
	
	-- Clear missing buffs tables
	bc.Buffs.Clear()
		
	-- Disable output
	BuffCheck_ActiveSettings.DisableBuffOutput = bc.Defaults.DisableBuffOutput
	bc.Btn.Settings.DisableBuffOutput:SetChecked(bc.Defaults.DisableBuffOutput)
		
	-- Announce to group
	BuffCheck_ActiveSettings.AnnounceMissing = bc.Defaults.AnnounceMissing
	bc.Btn.Settings.Announce:SetChecked(bc.Defaults.AnnounceMissing)
	
	-- Check after readycheck
	BuffCheck_ActiveSettings.CheckAfterReadycheck = bc.Defaults.CheckAfterReadycheck
	bc.Btn.Settings.Readycheck:SetChecked(bc.Defaults.CheckAfterReadycheck)

	-- Display settings on load
	BuffCheck_ActiveSettings.DisplaySettingsOnLoad = bc.Defaults.DisplaySettingsOnLoad
	bc.Btn.Settings.DisplaySettingsOnLoad:SetChecked(bc.Defaults.DisplaySettingsOnLoad)
	
	-- Consolidate output
	BuffCheck_ActiveSettings.ConsolidateOutput = bc.Defaults.ConsolidateOutput
	bc.Btn.Settings.ConsolidateOutput:SetChecked(bc.Defaults.ConsolidateOutput)
	
	-- ENABLED BUFFS --
	
	-- Set all to defaults
	--BuffCheck_ActiveSettings.BuffStatus = bc.Defaults.BuffStatus
	
	for i in pairs(BuffCheck_ActiveSettings.BuffStatus) do
		BuffCheck_ActiveSettings.BuffStatus[i] = bc.Defaults.BuffStatus[i]
	end
	
	bc.SetAllBuffCheckbuttons()
	
	bc.Display("Defaults set.") 
	
end

-- Send Addon Message
-- bc.SendVersionCheck = function()

	-- SendAddonMessage("BUFF_CHECK", "VersionCheck", "RAID")

-- end

-- EVENT HANDLERS --===========================================================

-------------------------------------------------------------------------------
-- Sets global defaults if none are specified in the settings.
-------------------------------------------------------------------------------
bc.InitEventFrame:SetScript("OnEvent", function(self, event, addon, ...)
	
	if event == "ADDON_LOADED" and addon == BuffCheck_Name then
		
		-- SAVED VARIABLE TABLE (*** is this necessary???)
		BuffCheck_ActiveSettings = BuffCheck_ActiveSettings or {} -- Used for settings/options
		
		-- Disable output?
		if BuffCheck_ActiveSettings.DisableBuffOutput == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.DisableBuffOutput = bc.Defaults.DisableBuffOutput
			bc.Btn.Settings.DisableBuffOutput:SetChecked(bc.Defaults.DisableBuffOutput)
		end
		-- Set state of checkbutton within interface options
		bc.Btn.Settings.DisableBuffOutput:SetChecked(BuffCheck_ActiveSettings.DisableBuffOutput)
		
		-- Announce to chat?
		if BuffCheck_ActiveSettings.AnnounceMissing == nil then
			-- Set saved settings to defaults
			BuffCheck_ActiveSettings.AnnounceMissing = bc.Defaults.AnnounceMissing
			bc.Btn.Settings.Announce:SetChecked(bc.Defaults.AnnounceMissing)
		end
		-- Set state of checkbutton within interface options
		bc.Btn.Settings.Announce:SetChecked(BuffCheck_ActiveSettings.AnnounceMissing)
		
		-- Check after readycheck?
		if BuffCheck_ActiveSettings.CheckAfterReadycheck == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.CheckAfterReadycheck = bc.Defaults.CheckAfterReadycheck
			bc.Btn.Settings.Readycheck:SetChecked(bc.Defaults.CheckAfterReadycheck)
		end
		-- Set state of checkbutton within interface options
		bc.Btn.Settings.Readycheck:SetChecked(BuffCheck_ActiveSettings.CheckAfterReadycheck)
		
		-- Display settings on load?
		if BuffCheck_ActiveSettings.DisplaySettingsOnLoad == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.DisplaySettingsOnLoad = bc.Defaults.DisplaySettingsOnLoad
			bc.Btn.Settings.DisplaySettingsOnLoad:SetChecked(bc.Defaults.DisplaySettingsOnLoad)
		end
		-- Set state of checkbutton within interface options
		bc.Btn.Settings.DisplaySettingsOnLoad:SetChecked(BuffCheck_ActiveSettings.DisplaySettingsOnLoad)
		
		-- Consolidate Output
		if BuffCheck_ActiveSettings.ConsolidateOutput == nil then
			-- Set active settings to defaults
			BuffCheck_ActiveSettings.ConsolidateOutput = bc.Defaults.ConsolidateOutput
			bc.Btn.Settings.ConsolidateOutput:SetChecked(bc.Defaults.ConsolidateOutput)
		end
		-- Set state of checkbutton within interface options
		bc.Btn.Settings.ConsolidateOutput:SetChecked(BuffCheck_ActiveSettings.ConsolidateOutput)
		
		-- Enabled buffs
		if BuffCheck_ActiveSettings.BuffStatus == nil then
			BuffCheck_ActiveSettings.BuffStatus = bc.Defaults.BuffStatus
		end
		
		bc.SetAllBuffCheckbuttons()

		-- Register addon message prefix
		--RegisterAddonMessagePrefix("BUFF_CHECK")
		
		--bc.SendVersionCheck()
		
		-- DISPLAY LOAD MESSAGE --
		bc.Display(bc.Description.Title.." v"..bc.Description.Version.." loaded.")
		
		if BuffCheck_ActiveSettings.DisplaySettingsOnLoad then
			bc.ViewSettings()
		end
	
	end -- if event and addon
	
end)

-- SCRIPTS FOR INTERFACE OPTIONS --============================================

-- *** Move these to another file???

-------------------------------------------------------------------------------
-- (1)
-- Changes the settings table to match the state of the 'disable buff output' checkbutton.
-------------------------------------------------------------------------------

bc.Btn.Settings.DisableBuffOutput:SetScript("OnClick", function ()

	if bc.Btn.Settings.DisableBuffOutput:GetChecked() then
		BuffCheck_ActiveSettings.DisableBuffOutput = true
	else
		BuffCheck_ActiveSettings.DisableBuffOutput = false
	end
end)

-------------------------------------------------------------------------------
-- (2)
-- Changes the settings table to match the state of the 'announce' checkbutton.
-------------------------------------------------------------------------------
bc.Btn.Settings.Announce:SetScript("OnClick", function ()
	
	if bc.Btn.Settings.Announce:GetChecked() then
		BuffCheck_ActiveSettings.AnnounceMissing = true
	else
		BuffCheck_ActiveSettings.AnnounceMissing = false
	end
	
end)

-------------------------------------------------------------------------------
-- (3)
-- Changes the settings table to match the state of the 'readycheck' checkbutton.
-------------------------------------------------------------------------------
bc.Btn.Settings.Readycheck:SetScript("OnClick", function ()
	
	if bc.Btn.Settings.Readycheck:GetChecked() then
		BuffCheck_ActiveSettings.CheckAfterReadycheck = true
	else
		BuffCheck_ActiveSettings.CheckAfterReadycheck = false
	end
	
end)

-------------------------------------------------------------------------------
-- (4)
-- Changes the settings table to match the state of the 'settings on load' checkbutton.
-------------------------------------------------------------------------------
bc.Btn.Settings.DisplaySettingsOnLoad:SetScript("OnClick", function ()
	
	if bc.Btn.Settings.DisplaySettingsOnLoad:GetChecked() then
		BuffCheck_ActiveSettings.DisplaySettingsOnLoad = true
	else
		BuffCheck_ActiveSettings.DisplaySettingsOnLoad = false
	end
	
end)

bc.Btn.Settings.ConsolidateOutput:SetScript("OnClick", function ()
	
	if bc.Btn.Settings.ConsolidateOutput:GetChecked() then
		BuffCheck_ActiveSettings.ConsolidateOutput = true
	else
		BuffCheck_ActiveSettings.ConsolidateOutput = false
	end
	
end)

-- Food
bc.Btn.Settings.Buffs.Food:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Food:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Food = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Food = false
	end

end)

-- GreaterFlasks
bc.Btn.Settings.Buffs.GreaterFlasks:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.GreaterFlasks:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.GreaterFlasks = true
	else
		BuffCheck_ActiveSettings.BuffStatus.GreaterFlasks = false
	end

end)

-- Stats
bc.Btn.Settings.Buffs.Stats:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Stats:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Stats = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Stats = false
	end

end)

-- Stam
bc.Btn.Settings.Buffs.Stam:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Stam:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Stam = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Stam = false
	end

end)

-- Ap
bc.Btn.Settings.Buffs.Ap:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Ap:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Ap = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Ap = false
	end

end)

-- Sp
bc.Btn.Settings.Buffs.Sp:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Sp:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Sp = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Sp = false
	end

end)

-- Mastery
bc.Btn.Settings.Buffs.Mastery:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Mastery:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Mastery = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Mastery = false
	end

end)

-- Haste
bc.Btn.Settings.Buffs.Haste:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Haste:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Haste = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Haste = false
	end

end)

-- Crit
bc.Btn.Settings.Buffs.Crit:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Crit:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Crit = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Crit = false
	end

end)

-- Multi
bc.Btn.Settings.Buffs.Multi:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Multi:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Multi = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Multi = false
	end

end)

-- Vers
bc.Btn.Settings.Buffs.Vers:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Vers:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Vers = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Vers = false
	end

end)

-- AugmentRune
bc.Btn.Settings.Buffs.Rune:SetScript("OnClick", function ()

	if bc.Btn.Settings.Buffs.Rune:GetChecked() then
		BuffCheck_ActiveSettings.BuffStatus.Rune = true
	else
		BuffCheck_ActiveSettings.BuffStatus.Rune = false
	end

end)

bc.Btn.Settings.EnableAllBuffs:SetScript("OnClick", function()

	local IsAllEnabled = true

	for i in pairs(bc.Btn.Settings.Buffs) do
		if not bc.Btn.Settings.Buffs[i]:GetChecked() then
			IsAllEnabled = false
		end
	end

	if not IsAllEnabled then	
		bc.SetAllBuffCheckbuttons(true)
	else
		bc.SetAllBuffCheckbuttons(false)
	end
		
		
end)

bc.panel.refresh = function()

	bc.SetAllBuffCheckbuttons()

end

-------------------------------------------------------------------------------
-- Enables functionality of 'Defaults' button in the interface options.
-------------------------------------------------------------------------------
bc.panel.default = function()
	
	bc.SetDefaults()
	
end